;****** Startup-File fr den STM32F103ZET6 auf dem Eigenbau-Eval-Board ******
;
; 1. Speicherbereiche:
; interner ROM:          0h .. 0007_FFFFh    (512 KB)
; interner RAM:  2000_0000h .. 2000_FFFFh    ( 64 KB)
; externer RAM:  6000_0000h .. 600F_FFFFh    (  1 MB) an NE1 (PD7)
; externes CPLD: 6400_0000h .. 6407_FFFFh    (256 x 16 Bildspeicher )

ExtRamBegin EQU  0x60000000
ExRAMEnd    EQU  0x600FFFFF
CgBWSP      EQU  0x64000000
CgBWSPEnd   EQU  0x6407FFFF



     EXPORT     CgBWSP
     EXPORT     CgBWSPEnd
     EXPORT     ExtRamBegin
     EXPORT     ExRAMEnd

     EXPORT     Errors
     EXPORT     ErrAddr
     EXPORT     kzi
     EXPORT     Kommandozeile
     EXPORT     Cpt
     EXPORT     von
     EXPORT     bis
     EXPORT     Manifest
     EXPORT     Kaltstart


; hier die allerntigsten Speicherbereiche fr den Monitor-Betrieb
Top_Ram           EQU     0x20010000      ; der interne RAM geht von 20000000 bis 2000FFFFH
Errors            EQU     Top_Ram - 4     ; Bitfeld zum Merken von Fehlern etc. und zugleich..
ErrAddr           EQU     Errors  - 4     ; Adresse des letzten Fehlers
Cpt               EQU     ErrAddr - 4     ; allg. Pointer
von               EQU     Cpt - 4         ; allg. Pointer
bis               EQU     von - 4         ; allg. Pointer
kzi               EQU     bis - 4         ; Index fr Kommandozeile
Kommandozeile     EQU     kzi   - 128     ; Speicherbereich fr serielle Kommunikation
Main_Stack        EQU     Kommandozeile - 4

StartOfRam        EQU     0x20000000
EndOfRam          EQU     Main_Stack - 1024

;********************************************************************

                PRESERVE8
                THUMB
                AREA    RESET, CODE, READONLY

;********************* Vektortafel und Kaltstart ***********************


; Vector Table Mapped to Address 0 at Reset

                DCD     Main_Stack                 ; Top of Stack
                DCD     Kaltstart                  ; Reset Handler
                DCD     NMI_Handler                ; NMI Handler
                DCD     HardFault_Handler          ; Hard Fault Handler
                DCD     MemManage_Handler          ; MPU Fault Handler
                DCD     BusFault_Handler           ; Bus Fault Handler
                DCD     UsageFault_Handler         ; Usage Fault Handler
                DCD     0                          ; Reserved
                DCD     0                          ; Reserved
                DCD     0                          ; Reserved
                DCD     0                          ; Reserved
                DCD     SVC_Handler                ; SVCall Handler
                DCD     DebugMon_Handler           ; Debug Monitor Handler
                DCD     0                          ; Reserved
                DCD     PendSV_Handler             ; PendSV Handler
                DCD     SysTick_Handler            ; SysTick Handler

; External Interrupts
                DCD     WWDG_IRQHandler            ; Window Watchdog
                DCD     PVD_IRQHandler             ; PVD through EXTI Line detect
                DCD     TAMPER_IRQHandler          ; Tamper
                DCD     RTC_IRQHandler             ; RTC
                DCD     FLASH_IRQHandler           ; Flash
                DCD     RCC_IRQHandler             ; RCC
                DCD     EXTI0_IRQHandler           ; EXTI Line 0
                DCD     EXTI1_IRQHandler           ; EXTI Line 1
                DCD     EXTI2_IRQHandler           ; EXTI Line 2
                DCD     EXTI3_IRQHandler           ; EXTI Line 3
                DCD     EXTI4_IRQHandler           ; EXTI Line 4
                DCD     DMA1_Channel1_IRQHandler   ; DMA1 Channel 1
                DCD     DMA1_Channel2_IRQHandler   ; DMA1 Channel 2
                DCD     DMA1_Channel3_IRQHandler   ; DMA1 Channel 3
                DCD     DMA1_Channel4_IRQHandler   ; DMA1 Channel 4
                DCD     DMA1_Channel5_IRQHandler   ; DMA1 Channel 5
                DCD     DMA1_Channel6_IRQHandler   ; DMA1 Channel 6
                DCD     DMA1_Channel7_IRQHandler   ; DMA1 Channel 7
                DCD     ADC1_2_IRQHandler          ; ADC1 & ADC2
                DCD     USB_HP_CAN1_TX_IRQHandler  ; USB High Priority or CAN1 TX
                DCD     USB_LP_CAN1_RX0_IRQHandler ; USB Low  Priority or CAN1 RX0
                DCD     CAN1_RX1_IRQHandler        ; CAN1 RX1
                DCD     CAN1_SCE_IRQHandler        ; CAN1 SCE
                DCD     EXTI9_5_IRQHandler         ; EXTI Line 9..5
                DCD     TIM1_BRK_IRQHandler        ; TIM1 Break
                DCD     TIM1_UP_IRQHandler         ; TIM1 Update
                DCD     TIM1_TRG_COM_IRQHandler    ; TIM1 Trigger and Commutation
                DCD     TIM1_CC_IRQHandler         ; TIM1 Capture Compare
                DCD     TIM2_IRQHandler            ; TIM2
                DCD     TIM3_IRQHandler            ; TIM3
                DCD     TIM4_IRQHandler            ; TIM4
                DCD     I2C1_EV_IRQHandler         ; I2C1 Event
                DCD     I2C1_ER_IRQHandler         ; I2C1 Error
                DCD     I2C2_EV_IRQHandler         ; I2C2 Event
                DCD     I2C2_ER_IRQHandler         ; I2C2 Error
                DCD     SPI1_IRQHandler            ; SPI1
                DCD     SPI2_IRQHandler            ; SPI2
                DCD     USART1_IRQHandler          ; USART1
                DCD     USART2_IRQHandler          ; USART2
                DCD     USART3_IRQHandler          ; USART3
                DCD     EXTI15_10_IRQHandler       ; EXTI Line 15..10
                DCD     RTCAlarm_IRQHandler        ; RTC Alarm through EXTI Line
                DCD     USBWakeUp_IRQHandler       ; USB Wakeup from suspend
                DCD     TIM8_BRK_IRQHandler        ; TIM8 Break
                DCD     TIM8_UP_IRQHandler         ; TIM8 Update
                DCD     TIM8_TRG_COM_IRQHandler    ; TIM8 Trigger and Commutation
                DCD     TIM8_CC_IRQHandler         ; TIM8 Capture Compare
                DCD     ADC3_IRQHandler            ; ADC3
                DCD     FSMC_IRQHandler            ; FSMC
                DCD     SDIO_IRQHandler            ; SDIO
                DCD     TIM5_IRQHandler            ; TIM5
                DCD     SPI3_IRQHandler            ; SPI3
                DCD     UART4_IRQHandler           ; UART4
                DCD     UART5_IRQHandler           ; UART5
                DCD     TIM6_IRQHandler            ; TIM6
                DCD     TIM7_IRQHandler            ; TIM7
                DCD     DMA2_Channel1_IRQHandler   ; DMA2 Channel1
                DCD     DMA2_Channel2_IRQHandler   ; DMA2 Channel2
                DCD     DMA2_Channel3_IRQHandler   ; DMA2 Channel3
                DCD     DMA2_Channel4_5_IRQHandler ; DMA2 Channel4 & Channel5


;
Manifest        DCB    "\r\nEgon Monitor auf STM32F103 (CORTEX M3)\r\n",0
                ALIGN


; Kaltstart: CPU hat SP bereits von Adresse 0 geladen
; wir exportieren __main, um dem Laden eines ganzen Rattenschwanzes
; von unbenutzten Bibliotheksfunktionen vorzubeugen.

Kaltstart      PROC
               IMPORT  main
               EXPORT  __main

__main
     LDR     R0, =Errors
     SUB     R1, R1, R1
     STR     R1, [R0]               ;  Fehlerbits lschen

     B       Restart

Soft_Reset


Warmstart
     LDR     R1, =1


Restart
     LDR     R0, =Errors
     LDR     R2,[R0]
     ORR     R2,R2,R1
     STR     R2,[R0]

     LDR     R0, = main
     BLX     R0
     B       Warmstart
     ENDP


; Exception Handlers
; fhren einen Restart mit Fehlerbit in Errors aus

NMI_Handler       PROC
                  EXPORT  NMI_Handler                [WEAK]
                  LDR     R1, =2
                  B       Restart
                  ENDP

HardFault_Handler PROC
                  EXPORT  HardFault_Handler          [WEAK]
                  LDR     R1, =4
                  B       Restart
                  ENDP

MemManage_Handler PROC
                  EXPORT  MemManage_Handler          [WEAK]
                  LDR     R1, =8
                  B       Restart
                  ENDP

BusFault_Handler  PROC
                  EXPORT  BusFault_Handler           [WEAK]
                  LDR     R1, =16
                  B       Restart
                  ENDP
UsageFault_Handler PROC
                  EXPORT  UsageFault_Handler         [WEAK]
                  LDR     R1, =32
                  B       Restart
                  ENDP
DebugMon_Handler  PROC
                  EXPORT  DebugMon_Handler           [WEAK]
                  LDR     R1, =64
                  B       Restart
                  ENDP
PendSV_Handler    PROC
                  EXPORT  PendSV_Handler             [WEAK]
                  LDR     R1, =128
                  B       Restart
                  ENDP

;SysTick_Handler PROC
;                EXPORT  SysTick_Handler            [WEAK]
;                B       .
;                ENDP
                IMPORT  SysTick_Handler

Default_Handler PROC

                EXPORT  WWDG_IRQHandler            [WEAK]
                EXPORT  PVD_IRQHandler             [WEAK]
                EXPORT  TAMPER_IRQHandler          [WEAK]
                EXPORT  RTC_IRQHandler             [WEAK]
                EXPORT  FLASH_IRQHandler           [WEAK]
                EXPORT  RCC_IRQHandler             [WEAK]
                EXPORT  EXTI0_IRQHandler           [WEAK]
                EXPORT  EXTI1_IRQHandler           [WEAK]
                EXPORT  EXTI2_IRQHandler           [WEAK]
                EXPORT  EXTI3_IRQHandler           [WEAK]
                EXPORT  EXTI4_IRQHandler           [WEAK]
                EXPORT  DMA1_Channel1_IRQHandler   [WEAK]
                EXPORT  DMA1_Channel2_IRQHandler   [WEAK]
                EXPORT  DMA1_Channel3_IRQHandler   [WEAK]
                EXPORT  DMA1_Channel4_IRQHandler   [WEAK]
                EXPORT  DMA1_Channel5_IRQHandler   [WEAK]
                EXPORT  DMA1_Channel6_IRQHandler   [WEAK]
                EXPORT  DMA1_Channel7_IRQHandler   [WEAK]
                EXPORT  ADC1_2_IRQHandler          [WEAK]
                EXPORT  USB_HP_CAN1_TX_IRQHandler  [WEAK]
                EXPORT  USB_LP_CAN1_RX0_IRQHandler [WEAK]
                EXPORT  CAN1_RX1_IRQHandler        [WEAK]
                EXPORT  CAN1_SCE_IRQHandler        [WEAK]
                EXPORT  EXTI9_5_IRQHandler         [WEAK]
                EXPORT  TIM1_BRK_IRQHandler        [WEAK]
                EXPORT  TIM1_UP_IRQHandler         [WEAK]
                EXPORT  TIM1_TRG_COM_IRQHandler    [WEAK]
                EXPORT  TIM1_CC_IRQHandler         [WEAK]
                EXPORT  TIM2_IRQHandler            [WEAK]
                EXPORT  TIM3_IRQHandler            [WEAK]
                EXPORT  TIM4_IRQHandler            [WEAK]
                EXPORT  I2C1_EV_IRQHandler         [WEAK]
                EXPORT  I2C1_ER_IRQHandler         [WEAK]
                EXPORT  I2C2_EV_IRQHandler         [WEAK]
                EXPORT  I2C2_ER_IRQHandler         [WEAK]
                EXPORT  SPI1_IRQHandler            [WEAK]
                EXPORT  SPI2_IRQHandler            [WEAK]
                EXPORT  USART1_IRQHandler          [WEAK]
                EXPORT  USART2_IRQHandler          [WEAK]
                EXPORT  USART3_IRQHandler          [WEAK]
                EXPORT  EXTI15_10_IRQHandler       [WEAK]
                EXPORT  RTCAlarm_IRQHandler        [WEAK]
                EXPORT  USBWakeUp_IRQHandler       [WEAK]
                EXPORT  TIM8_BRK_IRQHandler        [WEAK]
                EXPORT  TIM8_UP_IRQHandler         [WEAK]
                EXPORT  TIM8_TRG_COM_IRQHandler    [WEAK]
                EXPORT  TIM8_CC_IRQHandler         [WEAK]
                EXPORT  ADC3_IRQHandler            [WEAK]
                EXPORT  FSMC_IRQHandler            [WEAK]
                EXPORT  SDIO_IRQHandler            [WEAK]
                EXPORT  TIM5_IRQHandler            [WEAK]
                EXPORT  SPI3_IRQHandler            [WEAK]
                EXPORT  UART4_IRQHandler           [WEAK]
                EXPORT  UART5_IRQHandler           [WEAK]
                EXPORT  TIM6_IRQHandler            [WEAK]
                EXPORT  TIM7_IRQHandler            [WEAK]
                EXPORT  DMA2_Channel1_IRQHandler   [WEAK]
                EXPORT  DMA2_Channel2_IRQHandler   [WEAK]
                EXPORT  DMA2_Channel3_IRQHandler   [WEAK]
                EXPORT  DMA2_Channel4_5_IRQHandler [WEAK]

WWDG_IRQHandler
PVD_IRQHandler
TAMPER_IRQHandler
RTC_IRQHandler
FLASH_IRQHandler
RCC_IRQHandler
EXTI0_IRQHandler
EXTI1_IRQHandler
EXTI2_IRQHandler
EXTI3_IRQHandler
EXTI4_IRQHandler
DMA1_Channel1_IRQHandler
DMA1_Channel2_IRQHandler
DMA1_Channel3_IRQHandler
DMA1_Channel4_IRQHandler
DMA1_Channel5_IRQHandler
DMA1_Channel6_IRQHandler
DMA1_Channel7_IRQHandler
ADC1_2_IRQHandler
USB_HP_CAN1_TX_IRQHandler
USB_LP_CAN1_RX0_IRQHandler
CAN1_RX1_IRQHandler
CAN1_SCE_IRQHandler
EXTI9_5_IRQHandler
TIM1_BRK_IRQHandler
TIM1_UP_IRQHandler
TIM1_TRG_COM_IRQHandler
TIM1_CC_IRQHandler
TIM2_IRQHandler
TIM3_IRQHandler
TIM4_IRQHandler
I2C1_EV_IRQHandler
I2C1_ER_IRQHandler
I2C2_EV_IRQHandler
I2C2_ER_IRQHandler
SPI1_IRQHandler
SPI2_IRQHandler
USART1_IRQHandler
USART2_IRQHandler
USART3_IRQHandler
EXTI15_10_IRQHandler
RTCAlarm_IRQHandler
USBWakeUp_IRQHandler
TIM8_BRK_IRQHandler
TIM8_UP_IRQHandler
TIM8_TRG_COM_IRQHandler
TIM8_CC_IRQHandler
ADC3_IRQHandler
FSMC_IRQHandler
SDIO_IRQHandler
TIM5_IRQHandler
SPI3_IRQHandler
UART4_IRQHandler
UART5_IRQHandler
TIM6_IRQHandler
TIM7_IRQHandler
DMA2_Channel1_IRQHandler
DMA2_Channel2_IRQHandler
DMA2_Channel3_IRQHandler
DMA2_Channel4_5_IRQHandler
                  LDR     R1, =256
                  B       Restart
                  ENDP

;----------------------- Supervisor Calls ----------------

SVC_Handler     PROC
                EXPORT  SVC_Handler                [WEAK]
                PRESERVE8

                TST     LR,#4                   ; Called from Handler Mode?
                MRSNE   R12,PSP                 ; Yes, use PSP
                MOVEQ   R12,SP                  ; No, use MSP
                LDR     R12,[R12,#24]           ; Read Saved PC from Stack
                LDRH    R12,[R12,#-2]           ; Load Halfword
                BICS    R12,R12,#0xFF00         ; Extract SVC Number

                PUSH    {R4,LR}                 ; Save Registers
                LDR     LR,=SVC_Count
                LDR     LR,[LR]
                CMP     R12,LR
                BHS     SVC_Dead                ; Overflow
                LDR     LR,=SVC_Table
                LDR     R12,[LR,R12,LSL #2]     ; Load SVC Function Address
                BLX     R12                     ; Call SVC Function

                POP     {R4,LR}
                TST     LR,#4
                MRSNE   R12,PSP
                MOVEQ   R12,SP
                STM     R12,{R0-R3}             ; Function return values
                BX      LR                      ; RETI

; wenn SVC Nummer zu gro ist
SVC_Dead
                LDR     R1, =512
                B       Restart
                ALIGN

SVC_Cnt         EQU    (SVC_End-SVC_Table)/4
SVC_Count       DCD     SVC_Cnt

SVC_Table
                DCD  SVC_Dead                   ; 0   die ersten werden freigehalten
                DCD  SVC_Dead                   ; 1   wegen evtl. RTOS
                DCD  SVC_Dead                   ; 2
                DCD  SVC_Dead                   ; 3
                DCD  SVC_Dead                   ; 4
                DCD  SVC_Dead                   ; 5
                DCD  SVC_Dead                   ; 6
                DCD  SVC_Dead                   ; 7

; ab hier OS-Funktionen
                DCD       Soft_Reset                 ; 8 --> Restart

; Importe aus dem GDI
                IMPORT    CgFillScreen
                DCD       CgFillScreen          ; 9

                IMPORT    Heller
                DCD       Heller                ; 10

                IMPORT    Dunkler
                DCD       Dunkler               ; 11

                IMPORT    GetDC
                DCD       GetDC                 ; 12

                IMPORT    SelGFont
                DCD       SelGFont              ; 13

                IMPORT    CgPixel_at
                DCD       CgPixel_at            ; 14

                IMPORT    CgMoveTo
                DCD       CgMoveTo              ; 15

                IMPORT    CgLineTo
                DCD       CgLineTo              ; 16

                IMPORT    CgFillRect
                DCD       CgFillRect            ; 17

                IMPORT    CgCh_at
                DCD       CgCh_at               ; 18

                IMPORT    CgStr_at
                DCD       CgStr_at              ; 19

                IMPORT    StringWidth
                DCD       StringWidth           ; 20

                IMPORT    Get_dY
                DCD       Get_dY                ; 21

                IMPORT    Get_Ascent
                DCD       Get_Ascent            ; 22

                IMPORT    Get_Descent
                DCD       Get_Descent           ; 23

                IMPORT    Get_dX
                DCD       Get_dX                ; 24

                IMPORT    Get_Font
                DCD       Get_Font              ; 25

                IMPORT    DispatchEvent
                DCD       DispatchEvent         ; 26

                IMPORT    LocalToScreen
                DCD       LocalToScreen         ; 27

                IMPORT    Umrande
                DCD       Umrande               ; 28

                IMPORT    PanelKeyHandler
                DCD       PanelKeyHandler       ; 29

                IMPORT    PanelEventHandler
                DCD       PanelEventHandler     ; 30

                IMPORT    PanelDrawProc
                DCD       PanelDrawProc         ; 31

                IMPORT    ButtonDrawProc
                DCD       ButtonDrawProc        ; 32


                IMPORT    Messagebox
                DCD       Messagebox            ; 33

                IMPORT    GetSetMenu
                DCD       GetSetMenu            ; 34

                IMPORT    InitSerial
                DCD       InitSerial            ; 35

                IMPORT    numTxFree
                DCD       numTxFree             ; 36

                IMPORT    Char_Out
                DCD       Char_Out              ; 37

                IMPORT    String_Out
                DCD       String_Out            ; 38

                IMPORT    CRLF_Out
                DCD       CRLF_Out		; 39

                IMPORT    RxAvail
                DCD       RxAvail		; 40

                IMPORT    GetChar
                DCD       GetChar		; 41

                IMPORT    IsKeyAvailable
                DCD       IsKeyAvailable	; 42

                IMPORT    IsKeyPressed
                DCD       IsKeyPressed		; 43

                IMPORT    GetKey
                DCD       GetKey		; 44

                IMPORT    TranslateKeynum
                DCD       TranslateKeynum	; 45

                IMPORT    TastenZustand
                DCD       TastenZustand		; 46

                IMPORT    CgPicture_at
                DCD       CgPicture_at          ; 47

                IMPORT    CgCircle_at
                DCD       CgCircle_at           ; 48

                IMPORT    Ich_selected
                DCD       Ich_selected          ; 49

                IMPORT    StringCopy
                DCD       StringCopy            ; 50

                IMPORT    StringAdd
                DCD       StringAdd             ; 51

                IMPORT    Add_Delayed_Event
                DCD       Add_Delayed_Event     ; 52

                IMPORT    Add_Event
                DCD       Add_Event             ; 53

                IMPORT    Event_avail
                DCD       Event_avail           ; 54

                IMPORT    Get_Event
                DCD       Get_Event             ; 55

                IMPORT    PlaySound
                DCD       PlaySound             ; 56

                IMPORT    GetNibble
                DCD       GetNibble             ; 57

                IMPORT    Hex_In
                DCD       Hex_In                ; 58

                IMPORT    Long_In
                DCD       Long_In               ; 59

                IMPORT    Float_In_Ex
                DCD       Float_In_Ex           ; 60

                IMPORT    FF_round
                DCD       FF_round              ; 61

                IMPORT    UpCase
                DCD       UpCase                ; 62

                IMPORT    IgnoreSpace
                DCD       IgnoreSpace           ; 63

                IMPORT    match
                DCD       match                 ; 64

                IMPORT    Nibble_Out
                DCD       Nibble_Out            ; 65

                IMPORT    HexB_Out
                DCD       HexB_Out              ; 66

                IMPORT    HexW_Out
                DCD       HexW_Out              ; 67

                IMPORT    HexL_Out
                DCD       HexL_Out              ; 68

                IMPORT    HexQ_Out
                DCD       HexQ_Out              ; 69

                IMPORT    Dezi_Out
                DCD       Dezi_Out              ; 70

                IMPORT    FF_Out
                DCD       FF_Out                ; 71

                IMPORT    LONGtoStr
                DCD       LONGtoStr             ; 72

                IMPORT    FFtoStr_Ex
                DCD       FFtoStr_Ex            ; 73


SVC_End



                ALIGN



                ENDP




                ALIGN
                END

